@@ -68,20 +68,22 @@ char *WINAPI STRDUP(const char *c)
 // NB - the order of this list is important - see below
 
 #define FIELD_INVALID      (0)
-#define FIELD_LABEL        (1)
-#define FIELD_ICON         (2)
-#define FIELD_BITMAP       (3)
-#define FIELD_BROWSEBUTTON (4)
-#define FIELD_LINK         (5)
-#define FIELD_BUTTON       (6)
-#define FIELD_GROUPBOX     (7)
-#define FIELD_CHECKBOX     (8)
-#define FIELD_RADIOBUTTON  (9)
-#define FIELD_TEXT         (10)
-#define FIELD_FILEREQUEST  (11)
-#define FIELD_DIRREQUEST   (12)
-#define FIELD_COMBOBOX     (13)
-#define FIELD_LISTBOX      (14)
+#define FIELD_HLINE        (1)
+#define FIELD_VLINE        (2)
+#define FIELD_LABEL        (3)
+#define FIELD_ICON         (4)
+#define FIELD_BITMAP       (5)
+#define FIELD_BROWSEBUTTON (6)
+#define FIELD_LINK         (7)
+#define FIELD_BUTTON       (8)
+#define FIELD_GROUPBOX     (9)
+#define FIELD_CHECKBOX     (10)
+#define FIELD_RADIOBUTTON  (11)
+#define FIELD_TEXT         (12)
+#define FIELD_FILEREQUEST  (13)
+#define FIELD_DIRREQUEST   (14)
+#define FIELD_COMBOBOX     (15)
+#define FIELD_LISTBOX      (16)
 
 #define FIELD_SETFOCUS     FIELD_CHECKBOX // First field that qualifies for having the initial keyboard focus
 #define FIELD_CHECKLEN     FIELD_TEXT     // First field to have length of state value checked against MinLen/MaxLen
@@ -120,6 +122,7 @@ char *WINAPI STRDUP(const char *c)
 // WS_HSCROLL              0x00100000 // *ALL*
 // WS_VSCROLL              0x00200000 // *ALL*
 // WS_DISABLED             0x08000000 // *ALL*
+#define FLAG_FOCUS         0x10000000 // Controls that can receive focus
 
 struct TableEntry {
   char *pszName;
@@ -159,6 +162,8 @@ struct FieldType {
 
   int    nField; // field number in INI file
   char  *pszHwndEntry; // "HWND" or "HWND2"
+
+  long   wndProc; 
 };
 
 // initial buffer size.  buffers will grow as required.
@@ -246,7 +251,9 @@ bool INLINE ValidateFields() {
       if (((pField->nMaxLength > 0) && (nLength > pField->nMaxLength)) ||
          ((pField->nMinLength > 0) && (nLength < pField->nMinLength))) {
         if (pField->pszValidateText) {
-          MessageBox(hConfigWindow, pField->pszValidateText, NULL, MB_OK|MB_ICONWARNING);
+          char szTitle[1024];
+          GetWindowText(hMainWindow, szTitle, sizeof(szTitle));
+          MessageBox(hConfigWindow, pField->pszValidateText, szTitle, MB_OK|MB_ICONWARNING);
         }
         mySetFocus(pField->hwnd);
         return false;
@@ -452,6 +459,8 @@ int WINAPI ReadSettings(void) {
       { "LINK",        FIELD_LABEL       },
 #endif
       { "BUTTON",      FIELD_BUTTON      },
+      { "HLINE",       FIELD_HLINE       },
+      { "VLINE",       FIELD_VLINE       },
       { NULL,          0                 }
     };
     // Control flags
@@ -481,6 +490,7 @@ int WINAPI ReadSettings(void) {
       { "VSCROLL",           WS_VSCROLL          },
       { "DISABLED",          WS_DISABLED         },
       { "TRANSPARENT",       TRANSPARENT_BMP     },
+      { "FOCUS",             FLAG_FOCUS          },
       { NULL,                0                   }
     };
     FieldType *pField = pFields + nIdx;
@@ -520,7 +530,7 @@ int WINAPI ReadSettings(void) {
 
     // Label Text - convert newline
     pField->pszText = myGetProfileStringDup("TEXT");
-    if (pField->nType == FIELD_LABEL)
+    if (pField->nType == FIELD_LABEL || pField->nType == FIELD_LINK)
       ConvertNewLines(pField->pszText);
 
     // Dir request - root folder
@@ -581,112 +591,122 @@ int WINAPI ReadSettings(void) {
   return nNumFields;
 }
 
-
 LRESULT WINAPI WMCommandProc(HWND hWnd, UINT id, HWND hwndCtl, UINT codeNotify) {
-  switch (codeNotify) {
-    case BN_CLICKED:    // The user pressed a button
-    case LBN_SELCHANGE: // The user changed the selection in a ListBox control
-//  case CBN_SELCHANGE: // The user changed the selection in a DropList control (same value as LBN_SELCHANGE)
-    {
-      char szBrowsePath[MAX_PATH];
-      int nIdx = FindControlIdx(id);
-      // Ignore if the dialog is in the process of being created
-      if (g_done || nIdx < 0)
+  int nIdx = FindControlIdx(id);
+  // Ignore if the dialog is in the process of being created
+  if (g_done || nIdx < 0)
+    return 0;
+
+  switch (pFields[nIdx].nType)
+  {
+    case FIELD_BROWSEBUTTON:
+      --nIdx;
+    case FIELD_LINK:
+    case FIELD_BUTTON:
+    case FIELD_CHECKBOX:
+    case FIELD_RADIOBUTTON:
+      if (codeNotify != BN_CLICKED)
+        return 0;
+      break;
+    case FIELD_COMBOBOX:
+    case FIELD_LISTBOX:
+      if (codeNotify != LBN_SELCHANGE) // LBN_SELCHANGE == CBN_SELCHANGE
+        return 0;
+      break;
+    default:
+      return 0;
+  }
+
+  FieldType *pField = pFields + nIdx;
+
+  char szBrowsePath[MAX_PATH];
+
+  switch (pField->nType) {
+    case FIELD_FILEREQUEST: {
+      OPENFILENAME ofn={0,};
+
+      ofn.lStructSize = sizeof(ofn);
+      ofn.hwndOwner = hConfigWindow;
+      ofn.lpstrFilter = pField->pszFilter;
+      ofn.lpstrFile = szBrowsePath;
+      ofn.nMaxFile  = sizeof(szBrowsePath);
+      ofn.Flags = pField->nFlags & (OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_CREATEPROMPT | OFN_EXPLORER);
+
+      GetWindowText(pField->hwnd, szBrowsePath, sizeof(szBrowsePath));
+
+    tryagain:
+      GetCurrentDirectory(BUFFER_SIZE, szResult); // save working dir
+      if ((pField->nFlags & FLAG_SAVEAS) ? GetSaveFileName(&ofn) : GetOpenFileName(&ofn)) {
+        mySetWindowText(pField->hwnd, szBrowsePath);
+        SetCurrentDirectory(szResult); // restore working dir
+                                       // OFN_NOCHANGEDIR doesn't always work (see MSDN)
         break;
-      if (pFields[nIdx].nType == FIELD_BROWSEBUTTON)
-        --nIdx;
-      FieldType *pField = pFields + nIdx;
-      switch (pField->nType) {
-        case FIELD_FILEREQUEST: {
-          OPENFILENAME ofn={0,};
+      }
+      else if (szBrowsePath[0] && CommDlgExtendedError() == FNERR_INVALIDFILENAME) {
+        szBrowsePath[0] = '\0';
+        goto tryagain;
+      }
 
-          ofn.lStructSize = sizeof(ofn);
-          ofn.hwndOwner = hConfigWindow;
-          ofn.lpstrFilter = pField->pszFilter;
-          ofn.lpstrFile = szBrowsePath;
-          ofn.nMaxFile  = sizeof(szBrowsePath);
-          ofn.Flags = pField->nFlags & (OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_CREATEPROMPT | OFN_EXPLORER);
+      break;
+    }
 
-          GetWindowText(pField->hwnd, szBrowsePath, sizeof(szBrowsePath));
+    case FIELD_DIRREQUEST: {
+      BROWSEINFO bi;
 
-        tryagain:
-          GetCurrentDirectory(BUFFER_SIZE, szResult); // save working dir
-          if ((pField->nFlags & FLAG_SAVEAS) ? GetSaveFileName(&ofn) : GetOpenFileName(&ofn)) {
-            mySetWindowText(pField->hwnd, szBrowsePath);
-            SetCurrentDirectory(szResult); // restore working dir
-                                           // OFN_NOCHANGEDIR doesn't always work (see MSDN)
-            break;
-          }
-          else if (szBrowsePath[0] && CommDlgExtendedError() == FNERR_INVALIDFILENAME) {
-            szBrowsePath[0] = '\0';
-            goto tryagain;
-          }
-
-          break;
-        }
-
-        case FIELD_DIRREQUEST: {
-          BROWSEINFO bi;
-
-          bi.hwndOwner = hConfigWindow;
-          bi.pidlRoot = NULL;
-          bi.pszDisplayName = szBrowsePath;
-          bi.lpszTitle = pField->pszText;
+      bi.hwndOwner = hConfigWindow;
+      bi.pidlRoot = NULL;
+      bi.pszDisplayName = szBrowsePath;
+      bi.lpszTitle = pField->pszText;
 #ifndef BIF_NEWDIALOGSTYLE
 #define BIF_NEWDIALOGSTYLE 0x0040
 #endif
-          bi.ulFlags = BIF_STATUSTEXT | BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE;
-          bi.lpfn = BrowseCallbackProc;
-          bi.lParam = nIdx;
-          bi.iImage = 0;
+      bi.ulFlags = BIF_STATUSTEXT | BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE;
+      bi.lpfn = BrowseCallbackProc;
+      bi.lParam = nIdx;
+      bi.iImage = 0;
 
-          if (pField->pszRoot) {
-            LPSHELLFOLDER sf;
-            ULONG eaten;
-            LPITEMIDLIST root;
-            int ccRoot = (lstrlen(pField->pszRoot) * 2) + 2;
-            LPWSTR pwszRoot = (LPWSTR) MALLOC(ccRoot);
-            MultiByteToWideChar(CP_ACP, 0, pField->pszRoot, -1, pwszRoot, ccRoot);
-            SHGetDesktopFolder(&sf);
-            sf->ParseDisplayName(hConfigWindow, NULL, pwszRoot, &eaten, &root, NULL);
-            bi.pidlRoot = root;
-            sf->Release();
-            FREE(pwszRoot);
-          }
-//          CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
-          LPITEMIDLIST pResult = SHBrowseForFolder(&bi);
-          if (!pResult)
-            break;
+      if (pField->pszRoot) {
+        LPSHELLFOLDER sf;
+        ULONG eaten;
+        LPITEMIDLIST root;
+        int ccRoot = (lstrlen(pField->pszRoot) * 2) + 2;
+        LPWSTR pwszRoot = (LPWSTR) MALLOC(ccRoot);
+        MultiByteToWideChar(CP_ACP, 0, pField->pszRoot, -1, pwszRoot, ccRoot);
+        SHGetDesktopFolder(&sf);
+        sf->ParseDisplayName(hConfigWindow, NULL, pwszRoot, &eaten, &root, NULL);
+        bi.pidlRoot = root;
+        sf->Release();
+        FREE(pwszRoot);
+      }
+      //CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
+      LPITEMIDLIST pResult = SHBrowseForFolder(&bi);
+      if (!pResult)
+        break;
 
-          if (SHGetPathFromIDList(pResult, szBrowsePath)) {
-            mySetWindowText(pField->hwnd, szBrowsePath);
-          }
-
-          LPMALLOC pMalloc;
-          if (!SHGetMalloc(&pMalloc)) {
-            pMalloc->Free(pResult);
-          }
-
-          break;
-        }
-
-        case FIELD_LINK:
-        case FIELD_BUTTON:
-          // Allow the state to be empty - this might be useful in conjunction
-          // with the NOTIFY flag
-          if (*pField->pszState)
-            ShellExecute(hMainWindow, NULL, pField->pszState, NULL, NULL, SW_SHOWDEFAULT);
-          break;
+      if (SHGetPathFromIDList(pResult, szBrowsePath)) {
+        mySetWindowText(pField->hwnd, szBrowsePath);
       }
 
-      if (pField->nFlags & LBS_NOTIFY) {
-        // Remember which control was activated then pretend the user clicked Next
-        g_NotifyField = nIdx + 1;
-        mySendMessage(hMainWindow, WM_NOTIFY_OUTER_NEXT, 1, 0);
-      }
+      CoTaskMemFree(pResult);
+
+      break;
     }
-    break;
+
+    case FIELD_LINK:
+    case FIELD_BUTTON:
+      // Allow the state to be empty - this might be useful in conjunction
+      // with the NOTIFY flag
+      if (*pField->pszState)
+        ShellExecute(hMainWindow, NULL, pField->pszState, NULL, NULL, SW_SHOWDEFAULT);
+      break;
   }
+
+  if (pField->nFlags & LBS_NOTIFY) {
+    // Remember which control was activated then pretend the user clicked Next
+    g_NotifyField = nIdx + 1;
+    mySendMessage(hMainWindow, WM_NOTIFY_OUTER_NEXT, 1, 0);
+  }
+
   return 0;
 }
 
@@ -712,7 +732,7 @@ BOOL CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lPara
   if (message == WM_NOTIFY_OUTER_NEXT && !bRes)
   {
     // if leave function didn't abort (bRes != 0 in that case)
-    if (wParam == -1)
+    if (wParam == (WPARAM)-1)
       g_is_back++;
     if (wParam == NOTIFY_BYE_BYE)
       g_is_cancel++;
@@ -750,7 +770,7 @@ BOOL CALLBACK cfgDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       RECT rc = lpdis->rcItem;
 
       // Calculate needed size of the control
-      DrawText(lpdis->hDC, pField->pszText, -1, &rc, DT_VCENTER | DT_SINGLELINE | DT_CALCRECT);
+      DrawText(lpdis->hDC, pField->pszText, -1, &rc, DT_VCENTER | DT_WORDBREAK | DT_CALCRECT);
 
       // Make some more room so the focus rect won't cut letters off
       rc.right = min(rc.right + 2, lpdis->rcItem.right);
@@ -769,7 +789,7 @@ BOOL CALLBACK cfgDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
           SetTextColor(lpdis->hDC, (COLORREF) pField->hImage);
 
         // Draw the text
-        DrawText(lpdis->hDC, pField->pszText, -1, &rc, DT_CENTER | DT_VCENTER | DT_SINGLELINE | (bRTL ? DT_RTLREADING : 0));
+        DrawText(lpdis->hDC, pField->pszText, -1, &rc, DT_CENTER | DT_VCENTER | DT_WORDBREAK | (bRTL ? DT_RTLREADING : 0));
       }
 
       // Draw the focus rect if needed
@@ -858,6 +878,57 @@ int WINAPI StaticLINKWindowProc(HWND hWin, UINT uMsg, WPARAM wParam, LPARAM lPar
 }
 #endif
 
+int WINAPI NumbersOnlyPasteWndProc(HWND hWin, UINT uMsg, WPARAM wParam, LPARAM lParam)
+{
+  int nIdx = FindControlIdx(GetDlgCtrlID(hWin));
+  if (nIdx < 0)
+    return 0;
+
+  FieldType *pField = pFields + nIdx;
+
+  if (uMsg == WM_PASTE)
+  {
+    if (OpenClipboard(hWin))
+    {
+      HGLOBAL hData = GetClipboardData(CF_TEXT);
+      
+      if (hData)
+      {
+        char *lpData = (char *) GlobalLock(hData);
+        if (lpData)
+        {
+          int iLen = lstrlen(lpData);
+          char *lpFilteredData = (char *) MALLOC(iLen + 1);
+
+          if (lpFilteredData)
+          {
+            for (int i = 0, j = 0; i < iLen; i++)
+            {
+              if (lpData[i] >= '0' && lpData[i] <= '9')
+              {
+                lpFilteredData[j] = lpData[i];
+                j++;
+              }
+              lpFilteredData[j] = 0;
+            }
+
+            SendMessage(hWin, EM_REPLACESEL, TRUE, (LPARAM) lpFilteredData);
+            FREE(lpFilteredData);
+          }
+
+          GlobalUnlock(hData);
+        }
+      }
+
+      CloseClipboard();
+
+      return 0;
+    }
+  }
+
+  return CallWindowProc((WNDPROC) pField->wndProc, hWin, uMsg, wParam, lParam);
+}
+
 int old_cancel_visible;
 
 int WINAPI createCfgDlg()
@@ -897,7 +968,14 @@ int WINAPI createCfgDlg()
   mySetWindowText(hBackButton,pszBackButtonText);
 
   if (bBackEnabled!=-1) EnableWindow(hBackButton,bBackEnabled);
-  if (bCancelEnabled!=-1) EnableWindow(hCancelButton,bCancelEnabled);
+  if (bCancelEnabled!=-1)
+  {
+    EnableWindow(hCancelButton,bCancelEnabled);
+    if (bCancelEnabled)
+      EnableMenuItem(GetSystemMenu(mainwnd, FALSE), SC_CLOSE, MF_BYCOMMAND | MF_ENABLED);
+    else
+      EnableMenuItem(GetSystemMenu(mainwnd, FALSE), SC_CLOSE, MF_BYCOMMAND | MF_GRAYED);
+  }
   if (bCancelShow!=-1) old_cancel_visible=ShowWindow(hCancelButton,bCancelShow?SW_SHOWNA:SW_HIDE);
 
   HFONT hFont = (HFONT)mySendMessage(mainwnd, WM_GETFONT, 0, 0);
@@ -934,6 +1012,7 @@ int WINAPI createCfgDlg()
   }
 
   BOOL fFocused = FALSE;
+  BOOL fFocusedByFlag = FALSE;
 
 #define DEFAULT_STYLES (WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS)
 #define RTL_EX_STYLES (WS_EX_RTLREADING | WS_EX_LEFTSCROLLBAR)
@@ -946,6 +1025,16 @@ int WINAPI createCfgDlg()
       DWORD dwExStyle;
       DWORD dwRTLExStyle;
     } ClassTable[] = {
+      { "STATIC",       // FIELD_HLINE
+        DEFAULT_STYLES | SS_ETCHEDHORZ | SS_SUNKEN,
+        DEFAULT_STYLES | SS_ETCHEDHORZ | SS_SUNKEN,
+        WS_EX_TRANSPARENT,
+        WS_EX_TRANSPARENT | RTL_EX_STYLES },
+      { "STATIC",       // FIELD_VLINE
+        DEFAULT_STYLES | SS_ETCHEDVERT | SS_SUNKEN,
+        DEFAULT_STYLES | SS_ETCHEDVERT | SS_SUNKEN,
+        WS_EX_TRANSPARENT,
+        WS_EX_TRANSPARENT | RTL_EX_STYLES },
       { "STATIC",       // FIELD_LABEL
         DEFAULT_STYLES,
         DEFAULT_STYLES | SS_RIGHT,
@@ -1022,7 +1111,7 @@ int WINAPI createCfgDlg()
 
 #undef DEFAULT_STYLES
 
-    if (pField->nType < 1 || pField->nType > (sizeof(ClassTable) / sizeof(ClassTable[0])))
+    if (pField->nType < 1 || pField->nType > (int)(sizeof(ClassTable) / sizeof(ClassTable[0])))
       continue;
 
     DWORD dwStyle, dwExStyle;
@@ -1139,6 +1228,11 @@ int WINAPI createCfgDlg()
         case FIELD_FILEREQUEST:
         case FIELD_DIRREQUEST:
           mySendMessage(hwCtrl, EM_LIMITTEXT, (WPARAM)pField->nMaxLength, (LPARAM)0);
+          if (dwStyle & ES_NUMBER)
+          {
+            pField->wndProc = GetWindowLong(hwCtrl, GWL_WNDPROC);
+            SetWindowLong(hwCtrl, GWL_WNDPROC, (long) NumbersOnlyPasteWndProc);
+          }
           break;
 
         case FIELD_CHECKBOX:
@@ -1178,7 +1272,7 @@ int WINAPI createCfgDlg()
             FREE(pszList);
             if (pField->pszState) {
               if (pField->nFlags & (LBS_MULTIPLESEL|LBS_EXTENDEDSEL) && nFindMsg == LB_FINDSTRINGEXACT) {
-                mySendMessage(hwCtrl, LB_SETSEL, FALSE, -1);
+                mySendMessage(hwCtrl, LB_SETSEL, FALSE, (LPARAM)-1);
                 pszStart = pszEnd = pField->pszState;
                 for (;;) {
                   char c = *pszEnd;
@@ -1186,7 +1280,7 @@ int WINAPI createCfgDlg()
                     *pszEnd = '\0';
                     if (*pszStart)
                     {
-                      int nItem = mySendMessage(hwCtrl, LB_FINDSTRINGEXACT, -1, (LPARAM)pszStart);
+                      int nItem = mySendMessage(hwCtrl, LB_FINDSTRINGEXACT, (WPARAM)-1, (LPARAM)pszStart);
                       if (nItem != LB_ERR)
                         mySendMessage(hwCtrl, LB_SETSEL, TRUE, nItem);
                     }
@@ -1199,7 +1293,7 @@ int WINAPI createCfgDlg()
                 }
               }
               else {
-                int nItem = mySendMessage(hwCtrl, nFindMsg, -1, (LPARAM)pField->pszState);
+                int nItem = mySendMessage(hwCtrl, nFindMsg, (WPARAM)-1, (LPARAM)pField->pszState);
                 if (nItem != CB_ERR) { // CB_ERR == LB_ERR == -1
                   mySendMessage(hwCtrl, nSetSelMsg, nItem, 0);
                 }
@@ -1324,10 +1418,15 @@ int WINAPI createCfgDlg()
 #endif
       }
 
-      // Set initial focus to the first appropriate field
-      if (!fFocused && (dwStyle & (WS_TABSTOP | WS_DISABLED)) == WS_TABSTOP && pField->nType >= FIELD_SETFOCUS) {
-        fFocused = TRUE;
-        mySetFocus(hwCtrl);
+      // Set initial focus to the first appropriate field ( with FOCUS flag)
+      if (!fFocusedByFlag && (dwStyle & (WS_TABSTOP | WS_DISABLED)) == WS_TABSTOP && pField->nType >= FIELD_SETFOCUS) {
+        if (pField->nFlags & FLAG_FOCUS) {
+          fFocusedByFlag = TRUE;
+        }
+        if (!fFocused || fFocusedByFlag) {
+          fFocused = TRUE;
+          mySetFocus(hwCtrl);
+        }
       }
 
       // If multiline-readonly then hold the text back until after the
