@@ -5,7 +5,9 @@
 #include "Plugin.h"
 #include "Buffers.h"
 #include "System.h"
+#ifndef __GNUC__
 #include <crtdbg.h>
+#endif /* __GNUC__ */
 #include <objbase.h>
 
 // Parse Section Type 
@@ -39,6 +41,15 @@ HINSTANCE g_hInstance;
 char retexpr[4];
 HANDLE retaddr;
 
+#ifndef __GNUC__
+
+/*
+FIXME:
+GCC cannot compile the inline assembly used by System::Call and
+System::Get, so those functions and their supporting functions
+are disabled when using GCC.
+*/
+
 char *GetResultStr(SystemProc *proc)
 {
     char *buf = AllocString();
@@ -139,6 +150,12 @@ PLUGINFUNCTION(Debug)
 PLUGINFUNCTION(Get)
 {
     SystemProc *proc = PrepareProc(FALSE);
+    if (proc == NULL)
+    {
+      pushstring("error");
+      return;
+    }
+
     SYSTEM_LOG_ADD("Get ");
     SYSTEM_LOG_ADD(proc->DllName);
     SYSTEM_LOG_ADD("::");
@@ -169,6 +186,9 @@ PLUGINFUNCTION(Call)
 {
     // Prepare input
     SystemProc *proc = PrepareProc(TRUE);
+    if (proc == NULL)
+      return;
+
     SYSTEM_LOG_ADD("Call ");
     SYSTEM_LOG_ADD(proc->DllName);
     SYSTEM_LOG_ADD("::");
@@ -241,6 +261,8 @@ PLUGINFUNCTION(Call)
         GlobalFree((HANDLE) proc); // No, free it
 } PLUGINFUNCTIONEND
 
+#endif /* __GNUC__ */
+
 PLUGINFUNCTIONSHORT(Int64Op)
 {
     __int64 i1, i2 = 0, i3, i4;
@@ -298,6 +320,8 @@ __int64 GetIntFromString(char **p)
     return myatoi(buffer);
 }
 
+#ifndef __GNUC__
+
 SystemProc *PrepareProc(BOOL NeedForCall)
 {
     int SectionType = PST_PROC, // First section is always proc spec
@@ -320,6 +344,13 @@ SystemProc *PrepareProc(BOOL NeedForCall)
         // Check for Section Change
         BOOL changed = TRUE;
         ChangesDone = SectionType;
+
+        if (SectionType != PST_PROC && proc == NULL)
+            // no proc after PST_PROC is done means something is wrong with
+            // the call syntax and we'll get a crash because everything needs
+            // proc from here on.
+            break;
+
         switch (*ib)
         {
         case 0x0: SectionType = -1; break;
@@ -640,7 +671,7 @@ SystemProc *PrepareProc(BOOL NeedForCall)
     GlobalFree(sbuf);
 
     // Ok, the final step: check proc for existance
-    if (proc->Proc == NULL)
+    if (proc != NULL && proc->Proc == NULL)
     {
         switch (proc->ProcType)
         {
@@ -1354,6 +1385,8 @@ void CallStruct(SystemProc *proc)
     proc->Params[0].Value = (int) proc->Proc;
 }
 
+#endif /* __GNUC__ */
+
 BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
 {
         g_hInstance=hInst;
